{
 "cells": [
  {
   "cell_type": "markdown",
   "id": "f6f51d13-60e2-478c-8e11-1dd61e1cc330",
   "metadata": {},
   "source": [
    "# Plotly グラフのアニメーション\n",
    "\n",
    "公開日 <time datetime=\"2025-01-13\">2025-01-13</time>\n",
    "\n",
    "Pythonのインタラクティブなグラフを描画できるライブラリPlotlyにて、グラフのアニメーションを描画する方法を解説します。\n",
    "\n",
    "Plotlyには、高水準のAPIであるPlotly Expressと呼ばれるものもありますが、\n",
    "この記事では細かい調節ができる`graph_objects`を対象としています。"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "ab270676-1715-4118-aaf3-c890036ed80b",
   "metadata": {},
   "source": [
    "<!-- START MoshimoAffiliateEasyLink -->\n",
    "<script type=\"text/javascript\">\n",
    "(function(b,c,f,g,a,d,e){b.MoshimoAffiliateObject=a;\n",
    "b[a]=b[a]||function(){arguments.currentScript=c.currentScript\n",
    "||c.scripts[c.scripts.length-2];(b[a].q=b[a].q||[]).push(arguments)};\n",
    "c.getElementById(a)||(d=c.createElement(f),d.src=g,\n",
    "d.id=a,e=c.getElementsByTagName(\"body\")[0],e.appendChild(d))})\n",
    "(window,document,\"script\",\"//dn.msmstatic.com/site/cardlink/bundle.js?20220329\",\"msmaflink\");\n",
    "msmaflink({\"n\":\"pandas\\u0026Plotly 2D\\/3Dデータビジュアライゼーション実装ハンドブック\",\"b\":\"秀和システム\",\"t\":\"\",\"d\":\"https:\\/\\/m.media-amazon.com\",\"c_p\":\"\\/images\\/I\",\"p\":[\"\\/51buhQYpkrL._SL500_.jpg\",\"\\/41vnATKu5RL._SL500_.jpg\",\"\\/41vDFPPpE1L._SL500_.jpg\",\"\\/4150EmIK-VL._SL500_.jpg\",\"\\/41-G1+HL-lL._SL500_.jpg\",\"\\/41fwd4F77vL._SL500_.jpg\",\"\\/31LL+GnIWuL._SL500_.jpg\",\"\\/41XR6ul2enL._SL500_.jpg\",\"\\/41RCmqHCXuL._SL500_.jpg\",\"\\/41qtNs0Fe-L._SL500_.jpg\"],\"u\":{\"u\":\"https:\\/\\/www.amazon.co.jp\\/dp\\/479806890X\",\"t\":\"amazon\",\"r_v\":\"\"},\"v\":\"2.1\",\"b_l\":[{\"id\":1,\"u_tx\":\"Amazonで見る\",\"u_bc\":\"#f79256\",\"u_url\":\"https:\\/\\/www.amazon.co.jp\\/dp\\/479806890X\",\"a_id\":4672318,\"p_id\":170,\"pl_id\":27060,\"pc_id\":185,\"s_n\":\"amazon\",\"u_so\":1},{\"id\":2,\"u_tx\":\"楽天市場で見る\",\"u_bc\":\"#f76956\",\"u_url\":\"https:\\/\\/search.rakuten.co.jp\\/search\\/mall\\/pandas%26Plotly%202D%2F3D%E3%83%87%E3%83%BC%E3%82%BF%E3%83%93%E3%82%B8%E3%83%A5%E3%82%A2%E3%83%A9%E3%82%A4%E3%82%BC%E3%83%BC%E3%82%B7%E3%83%A7%E3%83%B3%E5%AE%9F%E8%A3%85%E3%83%8F%E3%83%B3%E3%83%89%E3%83%96%E3%83%83%E3%82%AF\\/\",\"a_id\":4672316,\"p_id\":54,\"pl_id\":27059,\"pc_id\":54,\"s_n\":\"rakuten\",\"u_so\":2}],\"eid\":\"IXitR\",\"s\":\"l\"});\n",
    "</script>\n",
    "<div id=\"msmaflink-IXitR\">リンク</div>\n",
    "<!-- MoshimoAffiliateEasyLink END -->"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "cbab4bd3-bb15-40c1-b463-ffb8d4454a64",
   "metadata": {},
   "source": [
    "## アニメーションの基礎\n",
    "\n",
    "棒グラフのアニメーションの簡単な例を示します。\n",
    "\"Play\"というボタンを押すと、棒グラフが伸びるアニメーションが再生されます。"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "555c2de0-bc3f-4eb2-9a59-14473f387b91",
   "metadata": {
    "scrolled": true
   },
   "outputs": [],
   "source": [
    "import plotly.graph_objects as go\n",
    "\n",
    "layout = go.Layout(\n",
    "    yaxis={\"range\": [0, 10]},\n",
    "    updatemenus=[\n",
    "        {\n",
    "            \"type\": \"buttons\",\n",
    "            \"buttons\": [\n",
    "                dict(label=\"Play\", method=\"animate\", args=[None]),\n",
    "            ],\n",
    "        },\n",
    "    ],\n",
    ")\n",
    "\n",
    "fig = go.Figure(\n",
    "    data=[go.Bar(x=[\"Alice\", \"Bob\"], y=[1, 2])],\n",
    "    layout=layout,\n",
    "    frames=[\n",
    "        go.Frame(data=[go.Bar(x=[\"Alice\", \"Bob\"], y=[1, 2])]),\n",
    "        go.Frame(data=[go.Bar(x=[\"Alice\", \"Bob\"], y=[2, 4])]),\n",
    "        go.Frame(data=[go.Bar(x=[\"Alice\", \"Bob\"], y=[3, 6])]),\n",
    "        go.Frame(data=[go.Bar(x=[\"Alice\", \"Bob\"], y=[4, 8])]),\n",
    "    ],\n",
    ")\n",
    "\n",
    "fig.show()"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "8eaf76d5-0d2a-4ac7-ae08-d6b7800ab757",
   "metadata": {},
   "source": [
    "上記の例を解説します。\n",
    "\n",
    "まず、以下の通り`go.Layout`の`updatemenus`属性にアニメーションを再生するボタンを設定します。\n",
    "\n",
    "- `label` (str): ボタンのラベル\n",
    "- `method` (str): アニメーションを再生するため、`\"animate\"`を指定\n",
    "- `args`: `[None]`とすると、通常のアニメーションとなります。\n",
    "\n",
    "```python\n",
    "layout = go.Layout(\n",
    "    yaxis={\"range\": [0, 10]},\n",
    "    updatemenus=[\n",
    "        {\n",
    "            \"type\": \"buttons\",\n",
    "            \"buttons\": [\n",
    "                dict(label=\"Play\", method=\"animate\", args=[None]),\n",
    "            ],\n",
    "        },\n",
    "    ],\n",
    ")\n",
    "```\n"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "e41e1fb8-5283-4612-810d-0bfa40c7a3ba",
   "metadata": {},
   "source": [
    "次に、`go.Figure`の`frames`属性に、アニメーションで表示するグラフを定義します。\n",
    "この`frames`属性には`go.Frame`インスタンスのリストを与えます。\n",
    "\n",
    "```python\n",
    "fig = go.Figure(\n",
    "    data=[go.Bar(x=[\"Alice\", \"Bob\"], y=[1, 2])],\n",
    "    layout=layout,\n",
    "    frames=[\n",
    "        go.Frame(data=[go.Bar(x=[\"Alice\", \"Bob\"], y=[1, 2])]),\n",
    "        go.Frame(data=[go.Bar(x=[\"Alice\", \"Bob\"], y=[2, 4])]),\n",
    "        go.Frame(data=[go.Bar(x=[\"Alice\", \"Bob\"], y=[3, 6])]),\n",
    "        go.Frame(data=[go.Bar(x=[\"Alice\", \"Bob\"], y=[4, 8])]),\n",
    "    ],\n",
    ")\n",
    "```"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "cc46caeb-66fe-4f65-927a-1414c26e4e6f",
   "metadata": {},
   "source": [
    "## アニメーションの再生速度\n",
    "\n",
    "アニメーションの再生速度を変更する場合、`updatemenus`の`args`に`{\"frame\": {\"duration\": 50}}`のような辞書を追加します。\n",
    "数値を大きくすると、ゆっくりと再生されます。\n",
    "\n",
    "```python\n",
    "layout = go.Layout(\n",
    "    xaxis={\"range\": [0, 2*np.pi]},\n",
    "    yaxis={\"range\": [-1.5, 1.5]},\n",
    "    updatemenus=[\n",
    "        {\n",
    "            \"type\": \"buttons\",\n",
    "            \"buttons\": [\n",
    "                dict(\n",
    "                    label=\"Play\",\n",
    "                    method=\"animate\",\n",
    "                    args=[None, {\"frame\": {\"duration\": 50}}]),\n",
    "            ],\n",
    "        },\n",
    "    ],\n",
    ")\n",
    "```\n"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "e7ba4c90-6aab-4e64-85a6-2a355a7d8568",
   "metadata": {},
   "source": [
    "## 複数プロットのアニメーション\n",
    "\n",
    "次に、複数のプロットに対するアニメーションを例示します。\n",
    "正弦波のグラフ上で点を動かすアニメーションです。\n"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "id": "790f86a4-983c-45ad-bc2f-b69284221249",
   "metadata": {},
   "outputs": [],
   "source": [
    "import numpy as np\n",
    "\n",
    "x = np.arange(start=0, stop=2*np.pi, step=0.1)\n",
    "sin_y = np.sin(x)\n",
    "\n",
    "layout = go.Layout(\n",
    "    xaxis={\"range\": [0, 2*np.pi]},\n",
    "    yaxis={\"range\": [-1.5, 1.5]},\n",
    "    updatemenus=[\n",
    "        {\n",
    "            \"type\": \"buttons\",\n",
    "            \"buttons\": [\n",
    "                dict(\n",
    "                    label=\"Play\",\n",
    "                    method=\"animate\",\n",
    "                    args=[None, {\"frame\": {\"duration\": 50}}]),\n",
    "            ],\n",
    "        },\n",
    "    ],\n",
    ")\n",
    "\n",
    "frames = [\n",
    "    go.Frame(\n",
    "        data=[go.Scatter(x=[x[i]], y=[sin_y[i]])],\n",
    "        traces=[1],\n",
    "    )\n",
    "    for i in range(len(x))\n",
    "]\n",
    "\n",
    "fig = go.Figure(\n",
    "    data=[\n",
    "        go.Scatter(x=x, y=sin_y, mode=\"lines\"),\n",
    "        go.Scatter(x=[x[0]], y=[sin_y[0]], mode=\"markers\", marker=dict(size=15)),\n",
    "    ],\n",
    "    layout=layout,\n",
    "    frames=frames,\n",
    ")\n",
    "\n",
    "fig.show()"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "e752132c-103a-4de8-988b-0238a3c0f7b6",
   "metadata": {},
   "source": [
    "変数`frames`では、リスト内包表記で`go.Frame`インスタンスのリストを作成しています。\n",
    "ここで`traces=[1]`としているのは、2つ目のグラフ（赤点のグラフ）を更新することを示しています。\n",
    "\n",
    "`go.Figure`の`data`には2つのグラフを定義しています。\n",
    "1つ目は青い折れ線グラフ(`mode=\"lines\"`)、2つ目は赤点のグラフ(`mode=\"markers\"`)です。\n"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "c3ffffa1-38f4-47db-8a03-6715cdca4f2a",
   "metadata": {},
   "source": [
    "## 参考サイト\n",
    "\n",
    "- [Intro to animations in Python](https://plotly.com/python/animations/)\n"
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3 (ipykernel)",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.12.4"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 5
}
